unit Main;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls;

type
  TForm1 = class(TForm)
    Image1: TImage;
    Button1: TButton;
    LabeledEdit1: TLabeledEdit;
    LabeledEdit2: TLabeledEdit;
    LabeledEdit3: TLabeledEdit;
    LabeledEdit4: TLabeledEdit;
    Memo1: TMemo;
    LabeledEdit5: TLabeledEdit;
    Button2: TButton;
    Timer1: TTimer;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private 錾 }
  public
    { Public 錾 }
    procedure drawArc(r, x, y, st, ed:double; w: integer; c: Tcolor);
    procedure drawPendulum(Q: double; F: boolean);
    procedure drawLine(xs, ys, xe, ye: double; w: integer; c: Tcolor);
    procedure drawtext(x, y: double; str: string; h: integer; Q: double);
    procedure ImageClear;
    function Elip1AGM(k: double): double;
    function calcQ(L, st, k: double): double;
    function Jacobi_sn(u, k: double): double;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  System.Math;

const
  G = 9.80665;                // d͉x

var
  x0, y0  : integer;          // image }_W
  T       : double;           // 
  AQ      : array of double;  // pxz
  N       : integer;          // 
  DBL_EPSILON : double;       // l
  TN      : integer;          // AjJE^[
  Q   : double;               // Upx
  ERF : boolean;              // AjtO
  QT  : double;               // Ajpx

// Jacobi֐ Jacobi sn(u, k)
function TForm1.Jacobi_sn(u, k: double): double;
const
  KN = 30;
var
  a: array of double;
  g: array of double;
  c: array of double;
  two_n   : integer;
  phi     : double;
  half    : double;
  i, j    : integer;
begin
  half := 1 / 2;
  if k = 0 then begin
    result := sin(u);
    exit;
  end;
  if k = 1 then begin
    result := sin(2 * arctan(exp(u)) - pi / 2);
    exit;
  end;
  setlength(a, KN + 1);
  setlength(g, KN + 1);
  setlength(c, KN + 1);
  a[0] := 1;
  g[0] := sqrt(1 - k * k);
  c[0] := k;
  two_n := 1;
  for i := 0 to KN do begin         // ȂKNŏI
    if abs(a[i] - g[i]) < a[i] * DBL_EPSILON then break;
    two_n := two_n + two_n;
    a[i + 1] := half * (a[i] + g[i]);
    g[i + 1] := sqrt(a[i] * g[i]);
    c[i + 1] := half * (a[i] - g[i]);
  end;
  phi := two_n * a[i] * u;
  for j := i downto 1 do
    phi := half * (phi + arcsin(c[j] * sin(phi) / a[j]));
  result := sin(phi);
end;

// 튮Sȉ~ϕ Zp􉽕
function TForm1.Elip1AGM(k: double): double;
var
  a, b, y : double;
  da, db  : double;
begin
  a := 1;
  b := sqrt(1 - k * k);
  da := a - b;
  db := 0;
  while da <> db do begin
    db := da;
    y := a;
    a := (a + b) / 2;
    b := sqrt(b * y);
    da := a - b;
  end;
  result := pi / 2 / a;
end;

// Uq̊pxvZ
// L Uq
// st 
// k S
function TForm1.calcQ(L, st, k: double): double;
var
  u, Q: double;
begin
  result := 0;
  if k >= 1 then exit;
  u := sqrt(G / L) * st;
  Q := k * Jacobi_sn(u, k);
  result := arcsin(Q) * 2 / pi * 180;
end;

// Arc }
// r   a
// x,y _ɑ΂W
// st, ed JnpIp@deg
// w ̕
// c  F
procedure TForm1.drawArc(r, x, y, st, ed:double; w: integer; c: Tcolor);
var
  xi, yi : integer;
  ri     : integer;
  xs, ys : integer;
  xe, ye : integer;
begin
  Image1.Canvas.Pen.Color := c;
  Image1.Canvas.Pen.Width := w;
  xi := x0 + round(x);
  yi := y0 - round(y);
  ri := round(r);
  xs := round(cos(st / 180 * pi) * r * 2) + xi;
  ys := yi - round(sin(st / 180 * pi) * r * 2);
  xe := round(cos(ed / 180 * pi) * r * 2) + xi;
  ye := yi - round(sin(ed / 180 * pi) * r * 2);
  Image1.Canvas.Arc(xi - ri, yi - ri, xi + ri + 1, yi + ri + 1, xs, ys, xe, ye);
end;

// Line }
// xs, ys n_W
// xe, ye I_W
// w      ̕
// c      ̐F
procedure TForm1.drawLine(xs, ys, xe, ye: double; w: integer; c: Tcolor);
var
  xsi, ysi : integer;
  xei, yei : integer;
begin
  Image1.Canvas.Pen.Color := c;
  Image1.Canvas.Pen.Width := w;
  xsi := x0 + round(xs);
  ysi := y0 - round(ys);
  xei := x0 + round(xe);
  yei := y0 - round(ye);
  Image1.Canvas.Pen.Color := c;
  Image1.Canvas.Pen.Width := w;
  Image1.Canvas.MoveTo(xsi, ysi);
  Image1.Canvas.LineTo(xei, yei);
end;

// pxo
// str  
// h    ̍
// Q    ̊px deg
procedure TForm1.drawtext(x, y: double; str: string; h: integer; Q: double);
var
  xi, yi : integer;
  lf, lfold: TLogFont;
begin
  xi := x0 + round(x);
  yi := y0 - round(y);
  //tHg̏擾
  GetObject(Image1.Canvas.Font.Handle, SizeOf(TLogFont), @lf);
  lfold := lf;
  lf.lfEscapement := round(Q * 10); // pxݒ
  lf.lfHeight := h;                 // 
  Image1.Canvas.Font.Handle := CreateFontIndirect(lf);
  Image1.Canvas.TextOut(xi, yi, str);
  Image1.Canvas.Font.Handle := CreateFontIndirect(lfOld);
end;

// Pendulum}
// Q px deg
// F true  false @}
procedure TForm1.drawPendulum(Q: double; F: boolean);
var
  qr : double;          // pxrad
  st, ed : double;      // JnpIp@deg
  L      : double;      // Pendulum
  x, y   : double;      // Pendulumʒu
  xs, ys : double;      // n_
  xe, ye : double;      // I_
begin
  qr := Q / 180 * pi;
  // x_~
  L := y0 - y0 / 3;
  // d~`
  st := 0 + Q ;
  ed := 180 + Q;
  x := L * sin(qr);
  y := - L * cos(qr);
  drawArc(50, x, y, ed - 59, st + 59, 2, clBlue);
  // o
  if not F then begin
    xe :=  (L - 5) * sin(qr - 0.3);
    ye := -(L - 5) * cos(qr - 0.3);
    drawtext(xe, ye,'Pendulum', 20,  Q);
  end
  else begin
    // dSʒuG`
    xe :=  (L - 20) * sin(qr + 0.07);
    ye := -(L - 20) * cos(qr + 0.07);
    image1.Canvas.Font.Color := clGreen;
    drawtext(xe, ye,'G', 20,  Q);
    // x_ʒuo`
    xe :=  (0 - 23) * sin(qr - 0.25);
    ye := -(0 - 23) * cos(qr - 0.25);
    image1.Canvas.Font.Color := clRed;
    drawtext(xe, ye,'o', 25,  Q);
    // x_ʒutodSʒuLine`
    xs := 5 * sin(qr);;
    ys := -5 * cos(qr);
    xe := (L - 15) * sin(qr);
    ye := -(L - 15) * cos(qr);
    drawLine(xs, ys, xe, ye, 1, clFuchsia);
    // dSʒu~`
    xs :=  (L - 10) * sin(qr);
    ys := -(L - 10) * cos(qr);
    drawArc(4, xs, ys, 0, 360, 2, clBlack);
    // dS̋h`
    xe := L / 2 * sin(qr - 0.1);;
    ye := -L / 2 * cos(qr - 0.1);
    drawtext(xe, ye,'h', 20,  Q);
    // dSʒu`
    ye := ys - 80;
    drawLine(xs, ys, xs, ye, 2, clBlack);
    xe := xs + 4;
    ys := ye + 14;
    drawLine(xs, ye, xe, ys, 2, clBlack);
    xe := xs - 4;
    drawLine(xs, ye, xe, ys, 2, clBlack);
    // ʂƉxMg`
    drawtext(xs - 10, ye,'Mg', 20,  0);
    // pxw_`
    xs := 0;
    ys := 0;
    xe := 0;
    ye := -120;
    drawLine(xs, ye, xe, ys, 1, clFuchsia);
    // pxw~ʕ`
    if Q > 5 then
      drawArc(110, 0, 0, 270, 270 + Q, 1, clFuchsia);
    // pxƕ`
    xs := (L - 28) * sin(qr - qr / 1.2);
    ys := -(L - 28) * cos(qr - qr / 1.2);
    drawtext(xs, ys,'', 20,  Q / 2);
    Image1.Canvas.Font.Color := clBlack;
    Image1.Canvas.Font.Height := 20;
    Image1.Canvas.TextOut(20, 20,'Ig  dSʒuG̊[g');
    Image1.Canvas.TextOut(20, 40,'M  S');
    Image1.Canvas.TextOut(20, 60,'h   x_OdSG̋');
    Image1.Canvas.TextOut(20, 80,'   Upx');
    Image1.Canvas.TextOut(20, 100,'g   d͉x 9.80665m/s^2');
  end;
  // ]S`
  drawArc(4, 0, 0, 0, 360, 2, clBlack);
  drawArc(25, 0, 0, st, ed, 2, clBlue);
  // bh`
  xs := 25 * cos(qr);
  ys := 25 * sin(qr);
  xe := L * sin(qr) + 50 * cos(qr + 1.05);
  ye := -L * cos(qr) - 50 * sin(-qr - 1.05);
  drawLine(xs, ys, xe, ye, 2, clBlue);
  xs := -25 * cos(qr);
  ys := -25 * sin(qr);
  xe := L * sin(qr) - 50 * cos(-qr + 1.05);
  ye := -L * cos(qr) - 50 * sin(qr - 1.05);
  drawLine(xs, ys, xe, ye, 2, clBlue);
end;

// 摜
procedure TForm1.ImageClear;
begin
  Image1.Canvas.Brush.Style := bsSolid;
  Image1.Canvas.Brush.Color := clWhite;
  Image1.Canvas.FillRect(Rect(0, 0, Image1.Width, Image1.Height));
end;

// ԁ@AjppxvZ
procedure TForm1.Button1Click(Sender: TObject);
var
  h   : double;   // x_dSʒu
  M   : double;   // 
  Ig  : double;   // [g
  Io  : double;   // [g
  L   : double;   // vZpUq
  ch  : integer;
  k   : double;   // S
  Fk  : double;   // 튮Sȉ~ϕl
  w   : double;
  dt, st  : double;
  i   : integer;
begin
  ImageClear;
  val(labelededit1.Text, h, ch);
  if ch <> 0 then begin
    application.MessageBox('x_dSʒuɊԈႢ܂B','',0);
    exit;
  end;
  if h <= 0 then begin
    application.MessageBox('x_dSʒu[̓[ȉłB','',0);
    exit;
  end;
  val(labelededit2.Text, M, ch);
  if ch <> 0 then begin
    application.MessageBox('ʂɊԈႢ܂B','',0);
    exit;
  end;
  if M <= 0 then begin
    application.MessageBox('ʂ[̓[ȉłB','',0);
    exit;
  end;
  val(labelededit3.Text, Ig, ch);
  if ch <> 0 then begin
    application.MessageBox('[gɊԈႢ܂B','',0);
    exit;
  end;
  if Ig <= 0 then begin
    application.MessageBox('[g[̓[ȉłB','',0);
    exit;
  end;
  val(labelededit4.Text, Q, ch);
  if ch <> 0 then begin
    application.MessageBox('UpxɊԈႢ܂B','',0);
    exit;
  end;
  if Q <= 0 then begin
    application.MessageBox('Upx[̓[ȉłB','',0);
    exit;
  end;
  if Q >= 180 then begin
    application.MessageBox('Upx傫܂180ȉɂĉB','',0);
    exit;
  end;
  val(labelededit5.Text, N, ch);
  if ch <> 0 then begin
    application.MessageBox('AjɊԈႢ܂B','',0);
    exit;
  end;
  if N < 10 then begin
    application.MessageBox('10ȉłB','',0);
    exit;
  end;
  drawPendulum(Q, true);                        // Uq}
  k  := sin(Q / 180 * pi / 2);                  // S
  Fk := Elip1AGM(k);                            // 튮Sȉ~ϕ
  Io := Ig + M * h * h;                         // ]S[g
  L := Io / M / h;                              // PUqvZ
//  T := 2 * pi * sqrt(Io / M / G / h);           // p
//  T := 4 * sqrt(Io / M / G / h) * Fk;           // 
  w := sqrt(L / G);                             // W
  T := 4 * w * Fk;                              // 
  memo1.Clear;
  memo1.Lines.Add('UqIo = ' + floatTostrF(Io, ffFixed, 10, 6));
  memo1.Lines.Add('PUq = ' + floatTostrF(L, ffFixed, 10, 6));
  memo1.Lines.Add('ԁ@@ = ' + floatTostrF(T, ffFixed, 10, 6));
  setlength(AQ, N + 1);                         // pxz̊m
  dt := T / N;                                  // t
  for i := 0 to N do begin                      // zpxvZ
    st := i * dt;
    AQ[i] := - calcQ(L, st, k);
  end;
  timer1.Interval := round(dt * 1000);          // ^C}[ݒ
  if timer1.Interval < 20 then timer1.Interval := 20;
  Button2.Enabled := True;                      // Aj{^Cl[u
  Button2.Caption := 'AjJn';
end;

// ^C}[荞
procedure TForm1.Timer1Timer(Sender: TObject);
begin
  if ERF then begin
    ERF := False;                               // tONA
    ImageClear;                                 // 摜
  end;
  if TN >= N then TN := 0;                      // zYNA
  QT := AQ[TN];                                 // px
  drawPendulum(Qt, False);                      // pxw`
  ERF := True;                                  // tOZbg
  inc(TN);                                      // zYCNg
end;

// Aj[VJn~
procedure TForm1.Button2Click(Sender: TObject);
begin
  ImageClear;
  ERF := False;
  if not Timer1.Enabled then begin
    TN := 0;
    Button1.Enabled := False;
    Timer1.Enabled := True;
    Button2.Caption := 'Aj~';
  end
  else begin
    Button1.Enabled := true;
    Timer1.Enabled := False;
    drawPendulum(Q, True);
    Button2.Caption := 'AjJn';
  end;
end;

// ݒ
procedure TForm1.FormCreate(Sender: TObject);
var
  y, z : double;
  i    : integer;
begin
  Timer1.Enabled := False;
  Button2.Enabled := False;
  X0 := Image1.Width div 2;                 // Uqx_WX
  y0 := Image1.Height div 2 - 20;           // Uqx_Wy
  memo1.Clear;
  drawPendulum(45, true);                   // 45}
  // lݒ
  i := 0;
  z := 1;
  DBL_EPSILON := 1;
  repeat
    DBL_EPSILON := DBL_EPSILON / 2;
    y := z +   DBL_EPSILON;
    inc(i);
  until (y = 1) or (i > 70);                // I
  DBL_EPSILON := DBL_EPSILON * 2;           // l
end;

end.
